@@ -1,4 +1,5 @@
 import asyncio
+from asyncio.futures import Future
 from typing import List
 from PIL import Image
 
@@ -7,12 +8,15 @@ from .remote.rustplus_pb2 import *
 from .remote import RustRemote, HeartBeat
 from ..commands import CommandOptions
 from ..exceptions import *
+from ..utils import RegisteredListener
 
 
 class BaseRustSocket:
 
-    def __init__(self, ip : str = None, port : str = None, steamid : int = None, playertoken : int = None, command_options : CommandOptions = None, raise_ratelimit_exception : bool = True, ratelimit_limit : int = 25, ratelimit_refill : int = 3, heartbeat : HeartBeat = None) -> None:
-        
+    def __init__(self, ip: str = None, port: str = None, steamid: int = None, playertoken: int = None,
+                 command_options: CommandOptions = None, raise_ratelimit_exception: bool = True,
+                 ratelimit_limit: int = 25, ratelimit_refill: int = 3, heartbeat: HeartBeat = None) -> None:
+
         if ip is None:
             raise ValueError("Ip cannot be None")
         if port is None:
@@ -29,14 +33,16 @@ class BaseRustSocket:
         self.seq = 1
         self.command_options = command_options
         self.raise_ratelimit_exception = raise_ratelimit_exception
+        self.listener_seq = 0
+
+        self.remote = RustRemote(ip=self.ip, port=self.port, command_options=command_options,
+                                 ratelimit_limit=ratelimit_limit, ratelimit_refill=ratelimit_refill)
 
-        self.remote = RustRemote(ip=self.ip, port=self.port, command_options=command_options, ratelimit_limit=ratelimit_limit, ratelimit_refill=ratelimit_refill)
-        
         if heartbeat is None:
             raise ValueError("Heartbeat cannot be None")
         self.heartbeat = heartbeat
 
-    async def _handle_ratelimit(self, amount = 1) -> None:
+    async def _handle_ratelimit(self, amount=1) -> None:
         """
         Handles the ratelimit for a specific request
         """
@@ -52,7 +58,7 @@ class BaseRustSocket:
                 raise RateLimitError("Out of tokens")
 
             await asyncio.sleep(self.remote.ratelimiter.get_estimated_delay_time(amount))
-    
+
     def _generate_protobuf(self) -> AppRequest:
         """
         Generates the empty AppRequest Message
@@ -103,8 +109,8 @@ class BaseRustSocket:
         self.remote.ignored_responses.append(app_request.seq)
 
         await self.remote.send_message(app_request)
-    
-    def command(self, coro) -> None:
+
+    def command(self, coro) -> RegisteredListener:
         """
         A Decorator to register a command listener
         """
@@ -112,21 +118,71 @@ class BaseRustSocket:
         if self.command_options is None:
             raise CommandsNotEnabledError("Not enabled")
 
-        self.remote.command_handler.registerCommand(coro.__name__, coro, asyncio.get_event_loop())
+        if isinstance(coro, RegisteredListener):
+            coro = coro.get_coro()
 
-    def team_event(self, coro) -> None:
+        data = (coro, asyncio.get_event_loop())
+        self.remote.command_handler.registerCommand(coro.__name__, data)
+        return RegisteredListener(coro.__name__, data)
+
+    def team_event(self, coro) -> RegisteredListener:
         """
         A Decorator to register an event listener for team changes
         """
 
-        self.remote.event_handler.register_event("team_changed", (coro, asyncio.get_event_loop()))
+        if isinstance(coro, RegisteredListener):
+            coro = coro.get_coro()
 
-    def chat_event(self, coro) -> None:
+        data = (coro, asyncio.get_event_loop())
+        self.remote.event_handler.register_event("team_changed", data)
+        return RegisteredListener("team_changed", data)
+
+    def chat_event(self, coro) -> RegisteredListener:
         """
         A Decorator to register an event listener for chat messages
         """
 
-        self.remote.event_handler.register_event("chat_message", (coro, asyncio.get_event_loop()))
+        if isinstance(coro, RegisteredListener):
+            coro = coro.get_coro()
+
+        data = (coro, asyncio.get_event_loop())
+        self.remote.event_handler.register_event("chat_message", data)
+        return RegisteredListener("chat_message", data)
+
+    def entity_event(self, eid):
+        """
+        Decorator to register a smart device listener
+        """
+
+        def wrap_func(coro) -> RegisteredListener:
+
+            if isinstance(coro, RegisteredListener):
+                coro = coro.get_coro()
+
+            def entity_event_callback(future: Future):
+                try:
+                    entity_info: RustEntityInfo = future.result()
+                    self.remote.event_handler.register_event(eid, (coro, loop, entity_info.type))
+                except:
+                    raise SmartDeviceRegistrationError("Not Found")
+
+            loop = asyncio.get_event_loop()
+            future = asyncio.run_coroutine_threadsafe(self.get_entity_info(eid), loop)
+            future.add_done_callback(entity_event_callback)
+
+            return RegisteredListener(eid, (coro))
+
+        return wrap_func
+
+    def remove_listener(self, listener) -> bool:
+        """
+        This will remove a listener, command or event. Takes a RegisteredListener instance
+        :returns Success of removal. True = Removed. False = Not Removed
+        """
+
+        if isinstance(listener, RegisteredListener):
+            return self.remote.remove_listener(listener)
+        return False
 
     async def hang(self) -> None:
         """
@@ -142,7 +198,7 @@ class BaseRustSocket:
         """
         raise NotImplementedError("Not Implemented")
 
-    async def send_team_message(self, message : str) -> None:
+    async def send_team_message(self, message: str) -> None:
         """
         Sends a message to the in-game team chat
         """
@@ -172,7 +228,8 @@ class BaseRustSocket:
         """
         raise NotImplementedError("Not Implemented")
 
-    async def get_map(self, add_icons : bool = False, add_events : bool = False, add_vending_machines : bool = False, override_images : dict = None) -> Image:
+    async def get_map(self, add_icons: bool = False, add_events: bool = False, add_vending_machines: bool = False,
+                      override_images: dict = None) -> Image:
         """
         Gets an image of the map from the server with the specified additions
         """
@@ -184,25 +241,25 @@ class BaseRustSocket:
         """
         raise NotImplementedError("Not Implemented")
 
-    async def get_entity_info(self, eid : int = None) -> RustEntityInfo:
+    async def get_entity_info(self, eid: int = None) -> RustEntityInfo:
         """
         Gets entity info from the server
         """
         raise NotImplementedError("Not Implemented")
 
-    async def turn_on_smart_switch(self, eid : int = None) -> None:
+    async def turn_on_smart_switch(self, eid: int = None) -> None:
         """
         Turns on a given smart switch by entity ID
         """
         raise NotImplementedError("Not Implemented")
 
-    async def turn_off_smart_switch(self, eid : int = None) -> None:
+    async def turn_off_smart_switch(self, eid: int = None) -> None:
         """
         Turns off a given smart switch by entity ID
         """
         raise NotImplementedError("Not Implemented")
 
-    async def promote_to_team_leader(self, steamid : int = None) -> None:
+    async def promote_to_team_leader(self, steamid: int = None) -> None:
         """
         Promotes a given user to the team leader by their 64-bit Steam ID
         """
@@ -220,7 +277,7 @@ class BaseRustSocket:
         """
         raise NotImplementedError("Not Implemented")
 
-    async def get_tc_storage_contents(self, eid : int = None, combine_stacks : bool = False) -> RustContents:
+    async def get_tc_storage_contents(self, eid: int = None, combine_stacks: bool = False) -> RustContents:
         """
         Gets the Information about TC Upkeep and Contents.
         Do not use this for any other storage monitor than a TC
