@@ -5,28 +5,30 @@ from datetime import datetime
 from . import Command, CommandTime
 from ..api.structures import RustChatMessage
 from ..commands.command_options import CommandOptions
+from ..utils import RegisteredListener
+
 
 class CommandHandler:
 
     def __init__(self, command_options : CommandOptions) -> None:
         self.command_options = command_options
 
-    def registerCommand(self, command, coro, loop) -> None:
+    def registerCommand(self, command, data) -> None:
         
-        if not asyncio.iscoroutinefunction(coro):
+        if not asyncio.iscoroutinefunction(data[0]):
             raise TypeError("The event registered must be a coroutine")
 
-        setattr(self, command, (coro, loop))
+        setattr(self, command, data)
 
     def _schedule_event(self, loop, coro, arg) -> None:
 
-        def callback(future : Future):
+        def callback(future: Future):
             future.result()
 
-        future : Future = asyncio.run_coroutine_threadsafe(coro(arg), loop)
+        future: Future = asyncio.run_coroutine_threadsafe(coro(arg), loop)
         future.add_done_callback(callback)
 
-    def run_command(self, message : RustChatMessage, prefix) -> None:
+    def run_command(self, message: RustChatMessage, prefix) -> None:
 
         if prefix == self.command_options.prefix:
             command = message.message.split(" ")[0][len(prefix):]
@@ -39,3 +41,9 @@ class CommandHandler:
             time = CommandTime(datetime.utcfromtimestamp(message.time), message.time)
 
             self._schedule_event(loop, coro, Command(message.name, message.steamId, time, command, message.message.split(" ")[1:]))
+
+    def has_command(self, listener: RegisteredListener) -> bool:
+        return hasattr(self, listener.listener_id)
+
+    def remove_command(self, listener: RegisteredListener) -> None:
+        delattr(self, listener.listener_id)
